@@ -13,9 +13,11 @@ from dnaio import (
     FileFormatError, FastaFormatError, FastqFormatError,
     FastaReader, FastqReader, InterleavedSequenceReader,
     FastaWriter, FastqWriter, InterleavedSequenceWriter,
-    PairedSequenceReader)
-from dnaio import _record_names_match, Sequence
-
+    PairedSequenceReader,
+)
+from dnaio import record_names_match, Sequence
+from dnaio.writers import FileWriter
+from dnaio.readers import BinaryFileReader
 
 # files tests/data/simple.fast{q,a}
 simple_fastq = [
@@ -110,9 +112,10 @@ class TestFastqReader:
         assert reads == simple_fastq
 
     def test_fastqreader_buffersize_too_small(self):
-        with raises(ValueError):
+        with raises(ValueError) as e:
             with FastqReader("tests/data/simple.fastq", buffer_size=0) as f:
                 _ = list(f)  # pragma: no cover
+        assert "buffer size too small" in e.value.args[0]
 
     def test_fastqreader_dos(self):
         # DOS line breaks
@@ -257,12 +260,15 @@ class TestOpen:
         assert reads == simple_fastq
 
         # make the name attribute unavailable
-        f = BytesIO(open("tests/data/simple.fastq", 'rb').read())
-        reads = list(dnaio.open(f))
+        with open("tests/data/simple.fastq", 'rb') as f:
+            data = f.read()
+        bio = BytesIO(data)
+        reads = list(dnaio.open(bio))
         assert reads == simple_fastq
-
-        f = BytesIO(open("tests/data/simple.fasta", 'rb').read())
-        reads = list(dnaio.open(f))
+        with open("tests/data/simple.fasta", 'rb') as f:
+            data = f.read()
+        bio = BytesIO(data)
+        reads = list(dnaio.open(bio))
         assert reads == simple_fasta
 
     def test_autodetect_fasta_format(self, tmpdir):
@@ -307,7 +313,8 @@ class TestOpen:
     def test_fastq_qualities_missing(self):
         path = os.path.join(self._tmpdir, 'tmp.fastq')
         with raises(ValueError):
-            dnaio.open(path, mode='w', qualities=False)
+            with dnaio.open(path, mode='w', qualities=False):
+                pass
 
 
 class TestInterleavedReader:
@@ -324,8 +331,6 @@ class TestInterleavedReader:
         ]
         with InterleavedSequenceReader("tests/data/interleaved.fastq") as isr:
             reads = list(isr)
-        for (r1, r2), (e1, e2) in zip(reads, expected):
-            print(r1, r2, e1, e2)
 
         assert reads == expected
         with dnaio.open("tests/data/interleaved.fastq", interleaved=True) as f:
@@ -463,7 +468,7 @@ class TestPairedSequenceReader:
             ] == list(psr)
 
     def test_record_names_match(self):
-        match = _record_names_match
+        match = record_names_match
         assert match('abc', 'abc')
         assert match('abc/1', 'abc/2')
         assert match('abc.1', 'abc.2')
@@ -509,10 +514,42 @@ def test_read_stdin(path):
         expected = len(list(f))
 
     # Use 'cat' to simulate that no file name is available for stdin of the subprocess
-    cat = subprocess.Popen(['cat', path], stdout=subprocess.PIPE)
-    py = subprocess.Popen(
-        [sys.executable, 'tests/read_from_stdin.py'], stdin=cat.stdout, stdout=subprocess.PIPE)
-    cat.stdout.close()
+    with subprocess.Popen(['cat', path], stdout=subprocess.PIPE) as cat:
+        with subprocess.Popen(
+            [sys.executable, 'tests/read_from_stdin.py'], stdin=cat.stdout, stdout=subprocess.PIPE
+        ) as py:
+            cat.stdout.close()
+            # Check that the read_from_stdin.py script prints the correct number of records
+            assert str(expected) == py.communicate()[0].decode().strip()
 
-    # Check that the read_from_stdin.py script prints the correct number of records
-    assert str(expected) == py.communicate()[0].decode().strip()
+
+def test_file_writer(tmp_path):
+    path = tmp_path / "out.txt"
+    fw = FileWriter(path)
+    repr(fw)
+    fw.close()
+    assert path.exists()
+    with raises(ValueError) as e:
+        with fw:
+            pass
+    assert "operation on closed file" in e.value.args[0]
+
+
+def test_binary_file_reader():
+    bfr = BinaryFileReader("tests/data/simple.fasta")
+    repr(bfr)
+    bfr.close()
+    with raises(ValueError) as e:
+        with bfr:
+            pass
+    assert "operation on closed" in e.value.args[0]
+
+
+def test_fasta_writer_repr(tmp_path):
+    with FastaWriter(tmp_path / "out.fasta") as fw:
+        repr(fw)
+
+
+def test_fastq_writer_repr(tmp_path):
+    with FastqWriter(tmp_path / "out.fastq") as fw:
+        repr(fw)
